#include <stdio.h> 
#include <signal.h>

#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/StripChart.h>
#include <X11/Xmu/SysUtil.h>

#include <stdlib.h>
#include "get_val.h"

#ifndef	DEBUG_XSTRIP
# define	DEBUG_XSTRIP	1
#endif

static struct generator	*genp;

void InitStripPoint(struct generator *gdp)
{
	char	*shell;

	if (!gdp) return;
	shell = getenv("SHELL");
	if (!shell) shell = "/bin/sh";
	genp = gdp;
	if (!gdp->cmd) {
		fprintf(stderr,"xstrip:  data generator is nil\n");
		exit(1);
	}
	if (pipe(gdp->inpipedesc) == -1 || pipe(gdp->outpipedesc) == -1) {
		perror("xstrip");
		fprintf(stderr,"xstrip:  no pipes to data generator\n");
		exit(1);
	}
	gdp->pid = fork();
	if (
#if	DEBUG_XSTRIP
	    0 ==
#endif
	    gdp->pid) {
		/*
		 * parent -- become data generator and thus allow the caller
		 * to know the data generator's exit status
		 *
		 * this code is incorrect if desc 0, 1 were available
		 * and the kernel used them for the pipe descriptors.
		 */
		(void) close(gdp->inpipedesc[0]);
		(void) close(gdp->outpipedesc[1]);
		(void) dup2(gdp->inpipedesc[1],1);
		(void) dup2(gdp->outpipedesc[0],0);
		(void) close(gdp->inpipedesc[1]);
		(void) close(gdp->outpipedesc[0]);
		execlp(shell,shell,"-c",gdp->cmd,0);
		exit(1);
	} else {
		(void) close(gdp->inpipedesc[1]);
		(void) close(gdp->outpipedesc[0]);
		gdp->in = fdopen(gdp->inpipedesc[0],"r");
	}
}

GetStripPoint(Widget wg, XtPointer p, XtPointer val)
{
	double			*vp;
	struct generator	*gdp;

	vp = (double *) val;
	gdp = (struct generator *) p;

	*vp = 0.0;
	if (!gdp) {
		if (scanf("%lf",vp) != 1)
			exit(0);
	} else {
		/* prompt for data */
		if (write(gdp->outpipedesc[1],"\n",1) != 1) {
			fprintf(stderr,"xstrip:  prompt to data generator failed\n");
			exit(1);
		}
		if (fscanf(gdp->in,"%lf",vp) != 1)
			exit(0);
	}
}
